from pydantic import BaseModel, Field

class ToolAgentOutput(BaseModel):
    """Standard output for all tool agents"""
    output: str
    sources: list[str] = Field(default_factory=list)

from .search_agent import search_agent
from .crawl_agent import crawl_agent

TOOL_AGENTS = {
    "WebSearchAgent": search_agent,
    "SiteCrawlerAgent": crawl_agent,
}


"""
Agent used to produce an initial outline of the report, including a list of section titles and the key question to be 
addressed in each section.

The Agent takes as input a string in the following format:
===========================================================
QUERY: <original user query>
===========================================================

The Agent then outputs a ReportPlan object, which includes:
1. A summary of initial background context (if needed), based on web searches and/or crawling
2. An outline of the report that includes a list of section titles and the key question to be addressed in each section
"""

from pydantic import BaseModel, Field
from typing import List
from agents import Agent
from ..llm_client import main_model
from .tool_agents.crawl_agent import crawl_agent
from .tool_agents.search_agent import search_agent
from datetime import datetime

class ReportPlanSection(BaseModel):
    """A section of the report that needs to be written"""
    title: str = Field(description="The title of the section")
    key_question: str = Field(description="The key question to be addressed in the section")


class ReportPlan(BaseModel):
    """Output from the Report Planner Agent"""
    background_context: str = Field(description="A summary of supporting context that can be passed onto the research agents")
    report_outline: List[ReportPlanSection] = Field(description="List of sections that need to be written in the report")


INSTRUCTIONS = f"""
You are a research manager, managing a team of research agents. Today's date is {datetime.now().strftime("%Y-%m-%d")}.
Given a research query, your job is to produce an initial outline of the report (section titles and key questions),
as well as some background context. Each section will be assigned to a different researcher in your team who will then
carry out research on the section.

You will be given:
- An initial research query

Your task is to:
1. Produce 1-2 paragraphs of initial background context (if needed) on the query by running web searches or crawling websites
2. Produce an outline of the report that includes a list of section titles and the key question to be addressed in each section. 

Guidelines:
- Each section should cover a single topic/question that is independent of other sections
- The key question for each section should include both the NAME and DOMAIN NAME / WEBSITE (if available and applicable) if it is related to a company, product or similar
- The background_context should not be more than 2 paragraphs
- The background_context should be very specific to the query and include any information that is relevant for researchers across all sections of the report
- The background_context should be draw only from web search or crawl results rather than prior knowledge (i.e. it should only be included if you have called tools)
- For example, if the query is about a company, the background context should include some basic information about what the company does
"""

    
planner_agent = Agent(
    name="PlannerAgent",
    instructions=INSTRUCTIONS,
    tools=[
        search_agent.as_tool(
            tool_name="web_search",
            tool_description="Use this tool to search the web for information relevant to the query - provide a query with 3-6 words as input"
        ),
        crawl_agent.as_tool(
            tool_name="crawl_website",
            tool_description="Use this tool to crawl a website for information relevant to the query - provide a starting URL as input"
        )
    ],
    model=main_model,
    output_type=ReportPlan,
)


import os
from openai import AsyncOpenAI
from agents import OpenAIChatCompletionsModel, OpenAIResponsesModel
from dotenv import load_dotenv

load_dotenv(override=True)

OPENAI_API_KEY = os.getenv("OPENAI_API_KEY")
DEEPSEEK_API_KEY = os.getenv("DEEPSEEK_API_KEY")
OPENROUTER_API_KEY = os.getenv("OPENROUTER_API_KEY")

DEFAULT_REASONING_PROVIDER=os.getenv("DEFAULT_REASONING_PROVIDER", "openai")
DEFAULT_REASONING_MODEL=os.getenv("DEFAULT_REASONING_MODEL", "o3-mini")
MAIN_MODEL_PROVIDER=os.getenv("MAIN_MODEL_PROVIDER", "openai")
MAIN_MODEL=os.getenv("MAIN_MODEL", "gpt-4o")
FAST_MODEL_PROVIDER=os.getenv("FAST_MODEL_PROVIDER", "openai")
FAST_MODEL=os.getenv("FAST_MODEL", "gpt-4o-mini")

supported_providers = ["openai", "deepseek", "openrouter"]
provider_mapping = {
    "openai": {
        "model": OpenAIResponsesModel,
        "base_url": None,
        "api_key": OPENAI_API_KEY,
    },
    "deepseek": {
        "model": OpenAIChatCompletionsModel,
        "base_url": "https://api.deepseek.com/v1",
        "api_key": DEEPSEEK_API_KEY,
    },
    "openrouter": {
        "model": OpenAIChatCompletionsModel,
        "base_url": "https://openrouter.ai/api/v1",
        "api_key": OPENROUTER_API_KEY,
    },
}

if DEFAULT_REASONING_PROVIDER not in supported_providers:
    raise ValueError(f"Invalid model provider: {DEFAULT_REASONING_PROVIDER}")
if MAIN_MODEL_PROVIDER not in supported_providers:
    raise ValueError(f"Invalid model provider: {MAIN_MODEL_PROVIDER}")
if FAST_MODEL_PROVIDER not in supported_providers:
    raise ValueError(f"Invalid model provider: {FAST_MODEL_PROVIDER}")

if OPENAI_API_KEY:
    from agents import set_tracing_export_api_key
    set_tracing_export_api_key(OPENAI_API_KEY)

# ------- SET UP REASONING MODEL -------

reasoning_client = AsyncOpenAI(
    api_key=provider_mapping[DEFAULT_REASONING_PROVIDER]["api_key"],
    base_url=provider_mapping[DEFAULT_REASONING_PROVIDER]["base_url"],
)

reasoning_model = provider_mapping[DEFAULT_REASONING_PROVIDER]["model"](
    model=DEFAULT_REASONING_MODEL,
    openai_client=reasoning_client
)

# ------- SET UP MAIN MODEL -------

main_client = AsyncOpenAI(
    api_key=provider_mapping[MAIN_MODEL_PROVIDER]["api_key"],
    base_url=provider_mapping[MAIN_MODEL_PROVIDER]["base_url"],
)

main_model = provider_mapping[MAIN_MODEL_PROVIDER]["model"](
    model=MAIN_MODEL,
    openai_client=main_client
)

# ------- SET UP FAST MODEL -------

fast_client = AsyncOpenAI(
    api_key=provider_mapping[FAST_MODEL_PROVIDER]["api_key"],
    base_url=provider_mapping[FAST_MODEL_PROVIDER]["base_url"],
)

fast_model = provider_mapping[FAST_MODEL_PROVIDER]["model"](
    model=FAST_MODEL,
    openai_client=fast_client
)

__all__ = ["reasoning_model", "main_model", "fast_model"]



"""
Agent used to crawl a website and return the results.

The SearchAgent takes as input a string in the format of AgentTask.model_dump_json(), or can take a simple starting url string as input

The Agent then:
1. Uses the crawl_website tool to crawl the website
2. Writes a 3+ paragraph summary of the crawled contents
3. Includes citations/URLs in brackets next to information sources
4. Returns the formatted summary as a string
"""

from agents import Agent
from ...tools import crawl_website
from . import ToolAgentOutput
from ...llm_client import fast_model


INSTRUCTIONS = """
You are a web craling agent that crawls the contents of a website answers a query based on the crawled contents. Follow these steps exactly:

* From the provided information, use the 'entity_website' as the starting_url for the web crawler
* Crawl the website using the crawl_website tool
* After using the crawl_website tool, write a 3+ paragraph summary that captures the main points from the crawled contents
* In your summary, try to comprehensively answer/address the 'gaps' and 'query' provided (if available)
* If the crawled contents are not relevant to the 'gaps' or 'query', simply write "No relevant results found"
* Use headings and bullets to organize the summary if needed
* Include citations/URLs in brackets next to all associated information in your summary
* Only run the crawler once
"""

crawl_agent = Agent(
    name="SiteCrawlerAgent",
    instructions="You are a site crawler agent that crawls a website and returns the results. ",
    tools=[crawl_website],
    model=fast_model,
    output_type=ToolAgentOutput,
)



"""
Agent used to perform web searches and summarize the results.

The SearchAgent takes as input a string in the format of AgentTask.model_dump_json(), or can take a simple query string as input

The Agent then:
1. Uses the web_search tool to retrieve search results
2. Analyzes the retrieved information
3. Writes a 3+ paragraph summary of the search results
4. Includes citations/URLs in brackets next to information sources
5. Returns the formatted summary as a string

The agent can use either OpenAI's built-in web search capability or a custom
web search implementation based on environment configuration.
"""

import os
from agents import Agent, WebSearchTool
from ...tools import web_search
from ...llm_client import fast_model
from dotenv import load_dotenv
from . import ToolAgentOutput

load_dotenv()
SEARCH_PROVIDER = os.getenv("SEARCH_PROVIDER", "serper").lower()

INSTRUCTIONS = """You are a research assistant. Given an AgentTask, follow these steps:

* Convert the 'query' into an optimized SERP search term for Google, limited to 3-5 words
* If an 'entity_website' is provided, make sure to include the domain name in your optimized Google search term
* Enter the optimized search term into the web_search tool
* After using the web_search tool, write a 3+ paragraph summary that captures the main points from the search results
* In your summary, try to comprehensively answer/address the 'gap' provided (which is the objective of the search)
* If the search results are not relevant to the search term or do not address the 'gap', simply write "No relevant results found"
* Use headings and bullets to organize the summary if needed
* Include citations/URLs in brackets next to all associated information in your summary
* Do not make additional searches
"""

if SEARCH_PROVIDER == "openai":
    web_search_tool = WebSearchTool()
else:
    web_search_tool = web_search

search_agent = Agent(
    name="WebSearchAgent",
    instructions=INSTRUCTIONS,
    tools=[web_search_tool],
    model=fast_model,
    output_type=ToolAgentOutput
)